/* Sample C program showing how to dynamically invoke the JVM (Java
 * interpreter) to call a Java method from C.
 *
 * Usage:  invokecc [ classpath ]
 *
 * The default classpath is listed below.  It must include the standard
 * Java library (classes.zip) as well as a directory containing the file
 * Example.class, which is the result of compiling a Java program such as the
 * following:
 *		public class Example {
 *			public static void exampleJavaMethod(String[] args) {
 *				System.out.println("Hello World" + args[0]);
 *			}
 *		}
 */
#include <jni.h>
#include <stdlib.h>
#include <assert.h>

char *classpath = CLASSPATH;

int main(int argc, char **argv) {
	JNIEnv *env;
	JavaVM *jvm;
	JDK1_1InitArgs vm_args;
	jint res;
	jclass cls, str_cls;
	jmethodID mid;
	jstring jstr;
	jobjectArray args;

	if (argc > 1)
		classpath = argv[1];

	/* Set up arguments for CreateJavaVM */
	JNI_GetDefaultJavaVMInitArgs(&vm_args);
	vm_args.classpath = classpath;

	/* Create a Java virtual machine and an environment for invoking it */
	res = JNI_CreateJavaVM(&jvm,(void **)&env,&vm_args);
	assert (res == 0);

	/* Get a handle on the Example class object */
	cls = (*env)->FindClass(env, "Example");
	assert (cls != 0);
 
	/* Look up the method Example.exampleJavaMethod.
	 * The third argument is a mangled version of the signature
	 *    void exampleJavaMethod(java.lang.String[]) :
	 *    (                            function with arguments
	 *       [                         arrray of
	 *          Ljava/lang/String;     class java.lang.String
	 *    )                            returning
	 *    V                            void
	 */
	mid = (*env)->GetStaticMethodID(env,
				cls, "exampleJavaMethod", "([Ljava/lang/String;)V");
	assert (mid != 0);

	/* Convert an ASCIZ string into a UTF string (Unicode represented using
	 * escape sequences rather than 16 bits/character).
	 */
	jstr = (*env)->NewStringUTF(env, " from C!");
	assert (jstr != 0);

	/* Get a handle on the class java.lang.String */
	str_cls = (*env)->FindClass(env, "java/lang/String");
	assert (str_cls != 0);

	/* Create an array containing one String serve as an arguement to
	 * Example.exampleJavaMethod.
	 */
	args = (*env)->NewObjectArray(env, 1, str_cls, jstr);
	assert (args != 0);


	/* Finally, call the method. */
	(*env)->CallStaticVoidMethod(env, cls, mid, args);

	(*jvm)->DestroyJavaVM(jvm);
	return 0;
}
