/* Sample C++ program showing how to dynamically invoke the JVM (Java
 * interpreter) to call a Java method from C++.
 *
 * Usage:  invokecc [ classpath ]
 *
 * The default classpath is listed below.  It must include the standard
 * Java library (classes.zip) as well as a directory containing the file
 * Example.class, which is the result of compiling a Java program such as the
 * following:
 *		public class Example {
 *			public static void exampleJavaMethod(String[] args) {
 *				System.out.println("Hello World" + args[0]);
 *			}
 *		}
 */
#include <jni.h>
#include <stdlib.h>
#include <assert.h>

char *classpath = CLASSPATH;

int main(int argc, char **argv) {
	if (argc > 1)
		classpath = argv[1];

	// Set up arguments for CreateJavaVM
	JDK1_1InitArgs vm_args;
	JNI_GetDefaultJavaVMInitArgs(&vm_args);
	vm_args.classpath = classpath;

	// Create a Java virtual machine and an environment for invoking it
	JavaVM *jvm;
	JNIEnv *env;
	jint res = JNI_CreateJavaVM(&jvm,(void **)&env,&vm_args);
	assert (res == 0);

	// Get a handle on the Example class object
	jclass cls = env->FindClass("Example");
	assert (cls != 0);

	// Look up the method Example.exampleJavaMethod.
	// The third argument is a mangled version of the signature
	//    void exampleJavaMethod(java.lang.String[]) :
	//    (                            function with arguments
	//       [                         arrray of
	//          Ljava/lang/String;     class java.lang.String
	//    )                            returning
	//    V                            void
	jmethodID mid =
		env->GetStaticMethodID(cls,
			"exampleJavaMethod", "([Ljava/lang/String;)V");
	assert (mid != 0);

	// Convert an ASCIZ string into a UTF string (Unicode represented using
	// escape sequences rather than 16 bits/character).
	jstring jstr = env->NewStringUTF(" from C++!");
	assert (jstr != 0);

	// Get a handle on the class java.lang.String
	jclass str_cls = env->FindClass("java/lang/String");
	assert (str_cls != 0);

	// Create an array containing one String serve as an arguement to
	// Example.exampleJavaMethod.
	jobjectArray args = env->NewObjectArray(1, str_cls, jstr);
	assert (args != 0);

	// Finally, call the method.
	env->CallStaticVoidMethod(cls, mid, args);

	jvm->DestroyJavaVM();
	return 0;
}
